package com.company.bugeventlistener.user

import com.company.bugeventlistener.entity.Child
import com.company.bugeventlistener.entity.GrandParent
import com.company.bugeventlistener.entity.Parent
import com.company.bugeventlistener.entity.User
import com.company.bugeventlistener.test_support.AuthenticatedAsAdmin
import io.jmix.core.DataManager
import io.jmix.core.security.UserRepository
import org.assertj.core.api.Assertions
import org.junit.jupiter.api.AfterEach
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.extension.ExtendWith
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.security.crypto.password.PasswordEncoder

@SpringBootTest
@ExtendWith(AuthenticatedAsAdmin::class)
class UserTest {
    @Autowired
    lateinit var dataManager: DataManager
    var savedUser: User? = null

    @Test
    fun test_events_works() {
        var grandParent = dataManager.create(GrandParent::class.java)

        grandParent.property = mutableListOf(dataManager.create(Parent::class.java))
        grandParent.property.first().grandParent = grandParent
        grandParent.property.first().property = dataManager.create(Child::class.java)
        val saveGrandParent = dataManager.save(grandParent)
        Assertions.assertThat(saveGrandParent.property.first().property?.data).isEqualTo("initialized by event listener")
    }

    @Test
    fun test_events_do_not_works() {
        var grandParent = dataManager.create(GrandParent::class.java)
        grandParent = dataManager.save(grandParent) // <<<-- only difference between the two tests
        grandParent.property = mutableListOf(dataManager.create(Parent::class.java))
        grandParent.property.first().grandParent = grandParent
        grandParent.property.first().property = dataManager.create(Child::class.java)
        val saveGrandParent = dataManager.save(grandParent)
        Assertions.assertThat(saveGrandParent.property.first().property?.data).isEqualTo("initialized by event listener")
    }
}